DataTypeLayout

Struct DataTypeLayout 

pub struct DataTypeLayout {
    pub buffers: Vec<BufferSpec>,
    pub can_contain_null_mask: bool,
    pub variadic: bool,
}
Layout specification for a data type

Fields

buffers: Vec<BufferSpec>

A vector of buffer layout specifications, one for each expected buffer

can_contain_null_mask: bool

Can contain a null bitmask

variadic: bool

This field only applies to the view type [DataType::BinaryView] and [DataType::Utf8View] If variadic is true, the number of buffers expected is only lower-bounded by buffers.len(). Buffers that exceed the lower bound are legal.

Implementations

Source

impl DataTypeLayout

pub fn new_fixed_width<T>() -> Self

Describes a basic numeric array where each element has type T

pub fn new_nullable_empty() -> Self

Describes arrays which have no data of their own but may still have a Null Bitmap (e.g. FixedSizeList)

pub fn new_empty() -> Self

Describes arrays which have no data of their own (e.g. RunEndEncoded).

pub fn new_binary<T>() -> Self

Describes a basic numeric array where each element has a fixed with offset buffer of type T, followed by a variable width data buffer

pub fn new_view() -> Self

Describes a view type

pub fn new_list_view<T>() -> Self

Describes a list view type