fn utf8_span<'a, T: ArrowNativeType>(
offsets: &[T],
values: &'a [u8],
) -> Option<(usize, usize, &'a str)>Expand description
Returns the first and last of offsets, and the bytes of values between
them as a str. Returns None if the offsets are not valid indexes into
values or those bytes are not valid UTF-8.